#! /usr/bin/env python

# Panflute
# Copyright (C) 2009 Paul Kuliniewicz <paul@kuliniewicz.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301, USA.

"""
Interface translator for VLC.

Little more than a simple passthrough.
"""

import panflute.daemon.passthrough

import gobject


class Connector (panflute.daemon.passthrough.Connector):
    """
    Connection manager for VLC.
    """

    def __init__ (self):
        panflute.daemon.passthrough.Connector.__init__ (self, "vlc", "VLC",
                                                        "vlc")
        self.props.icon_name = "vlc"


    def player (self, **kwargs):
        return Player (**kwargs)


class Player (panflute.daemon.passthrough.Player):
    """
    Player interface for VLC.

    Tries harder to fetch album art, since VLC doesn't always include it in
    the TrackChanged signal.
    """

    from panflute.util import log

    ART_FETCH_DELAY = 3000


    def __init__ (self, **kwargs):
        panflute.daemon.passthrough.Player.__init__ (self, "vlc", True, **kwargs)


    def do_TrackChange (self, metadata):
        if metadata is not None and metadata.has_key ("location") and not metadata.has_key ("arturl"):
            gobject.timeout_add (self.ART_FETCH_DELAY, self.__check_for_art)
        panflute.daemon.passthrough.Player.do_TrackChange (self, metadata)


    def __check_for_art (self):
        """
        Fetch the current metadata and report a change if there's now art
        available.
        """

        self.log.debug ("Checking for delayed art")
        new_metadata = self.do_GetMetadata ()
        if new_metadata.has_key ("arturl"):
            self.do_TrackChange (new_metadata)


    def _normalize_metadata (self, metadata):
        """
        Compensate for how VLC uses "title" for the stream name and "nowplaying"
        for the song currently being streamed.
        """

        if metadata is not None:
            if metadata.has_key ("nowplaying"):
                metadata["album"] = metadata["title"]
                metadata["title"] = metadata["nowplaying"]
                del metadata["nowplaying"]

        return panflute.daemon.passthrough.Player._normalize_metadata (self, metadata)
